@@ -6,7 +6,6 @@
 from __future__ import with_statement
 
 import errno
-import logging
 import os
 import select
 import signal
@@ -14,6 +13,8 @@ import sys
 import time
 import traceback
 
+
+from gunicorn.glogging import Logger
 from gunicorn.errors import HaltServer
 from gunicorn.pidfile import Pidfile
 from gunicorn.sock import create_socket
@@ -51,9 +52,6 @@ class Arbiter(object):
     )
     
     def __init__(self, app):
-        self.log = logging.getLogger(__name__)
-        self.log.info("Starting gunicorn %s" % __version__)
-       
         os.environ["SERVER_SOFTWARE"] = SERVER_SOFTWARE
 
         self.setup(app)
@@ -87,6 +85,11 @@ class Arbiter(object):
     def setup(self, app):
         self.app = app
         self.cfg = app.cfg
+        self.log = Logger(app.cfg)
+    
+        if 'GUNICORN_FD' in os.environ:
+            self.log.reopen_files()
+        
         self.address = self.cfg.address
         self.num_workers = self.cfg.workers
         self.debug = self.cfg.debug
@@ -97,7 +100,7 @@ class Arbiter(object):
         if self.cfg.debug:
             self.log.debug("Current configuration:")
             for config, value in sorted(self.cfg.settings.iteritems()):
-                self.log.debug("  %s: %s" % (config, value.value))
+                self.log.debug("  %s: %s", config, value.value)
         
         if self.cfg.preload_app:
             if not self.cfg.debug:
@@ -109,17 +112,22 @@ class Arbiter(object):
         """\
         Initialize the arbiter. Start listening and set pidfile if needed.
         """
+        self.log.info("Starting gunicorn %s", __version__)
+        self.cfg.on_starting(self)
         self.pid = os.getpid()
         self.init_signals()
         if not self.LISTENER:
-            self.LISTENER = create_socket(self.cfg)
+            self.LISTENER = create_socket(self.cfg, self.log)
         
         if self.cfg.pidfile is not None:
             self.pidfile = Pidfile(self.cfg.pidfile)
             self.pidfile.create(self.pid)
         self.log.debug("Arbiter booted")
-        self.log.info("Listening at: %s (%s)" % (self.LISTENER,
-            self.pid))
+        self.log.info("Listening at: %s (%s)", self.LISTENER,
+            self.pid)
+        self.log.info("Using worker: %s",
+                self.cfg.settings['worker_class'].get())
+
         self.cfg.when_ready(self)
     
     def init_signals(self):
@@ -128,10 +136,11 @@ class Arbiter(object):
         are queued. Child signals only wake up the master.
         """
         if self.PIPE:
-            map(lambda p: os.close(p), self.PIPE)
+            map(os.close, self.PIPE)
         self.PIPE = pair = os.pipe()
         map(util.set_non_blocking, pair)
         map(util.close_on_exec, pair)
+        self.log.close_on_exec()
         map(lambda s: signal.signal(s, self.signal), self.SIGNALS)
         signal.signal(signal.SIGCHLD, self.handle_chld)
 
@@ -140,7 +149,7 @@ class Arbiter(object):
             self.SIG_QUEUE.append(sig)
             self.wakeup()
         else:
-            self.log.warn("Dropping signal: %s" % sig)
+            self.log.warn("Dropping signal: %s", sig)
 
     def run(self):
         "Main master loop."
@@ -159,15 +168,15 @@ class Arbiter(object):
                     continue
                 
                 if sig not in self.SIG_NAMES:
-                    self.log.info("Ignoring unknown signal: %s" % sig)
+                    self.log.info("Ignoring unknown signal: %s", sig)
                     continue
                 
                 signame = self.SIG_NAMES.get(sig)
                 handler = getattr(self, "handle_%s" % signame, None)
                 if not handler:
-                    self.log.error("Unhandled signal: %s" % signame)
+                    self.log.error("Unhandled signal: %s", signame)
                     continue
-                self.log.info("Handling signal: %s" % signame)
+                self.log.info("Handling signal: %s", signame)
                 handler()  
                 self.wakeup()
             except StopIteration:
@@ -179,7 +188,7 @@ class Arbiter(object):
             except SystemExit:
                 raise
             except Exception:
-                self.log.info("Unhandled exception in main loop:\n%s" %  
+                self.log.info("Unhandled exception in main loop:\n%s",  
                             traceback.format_exc())
                 self.stop(False)
                 if self.pidfile is not None:
@@ -198,7 +207,7 @@ class Arbiter(object):
         - Start the new worker processes with a new configuration
         - Gracefully shutdown the old worker processes
         """
-        self.log.info("Hang up: %s" % self.master_name)
+        self.log.info("Hang up: %s", self.master_name)
         self.reload()
         
     def handle_quit(self):
@@ -239,6 +248,7 @@ class Arbiter(object):
         Kill all workers by sending them a SIGUSR1
         """
         self.kill_workers(signal.SIGUSR1)
+        self.log.reopen_files()
     
     def handle_usr2(self):
         """\
@@ -271,9 +281,9 @@ class Arbiter(object):
     def halt(self, reason=None, exit_status=0):
         """ halt arbiter """
         self.stop()
-        self.log.info("Shutting down: %s" % self.master_name)
+        self.log.info("Shutting down: %s", self.master_name)
         if reason is not None:
-            self.log.info("Reason: %s" % reason)
+            self.log.info("Reason: %s", reason)
         if self.pidfile is not None:
             self.pidfile.unlink()
         sys.exit(exit_status)
@@ -306,6 +316,10 @@ class Arbiter(object):
         :attr graceful: boolean, If True (the default) workers will be
         killed gracefully  (ie. trying to wait for the current connection)
         """
+        try:
+            self.LISTENER.close()
+        except Exception:
+            pass
         self.LISTENER = None
         sig = signal.SIGQUIT
         if not graceful:
@@ -323,17 +337,17 @@ class Arbiter(object):
         """
         if self.pidfile is not None:
             self.pidfile.rename("%s.oldbin" % self.pidfile.fname)
-        
+
         self.reexec_pid = os.fork()
         if self.reexec_pid != 0:
             self.master_name = "Old Master"
             return
-            
+
         os.environ['GUNICORN_FD'] = str(self.LISTENER.fileno())
         os.chdir(self.START_CTX['cwd'])
         self.cfg.pre_exec(self)
         os.execvpe(self.START_CTX[0], self.START_CTX['args'], os.environ)
-        
+
     def reload(self):
         old_address = self.cfg.address
 
@@ -345,11 +359,10 @@ class Arbiter(object):
         if old_address != self.cfg.address:
             self.LISTENER.close()
             self.LISTENER = create_socket(self.cfg)
-            self.log.info("Listening at: %s" % self.LISTENER)    
+            self.log.info("Listening at: %s", self.LISTENER)    
 
         # spawn new workers with new app & conf
-        for i in range(self.app.cfg.workers):
-            self.spawn_worker()
+        self.cfg.on_reload(self)
         
         # unlink pidfile
         if self.pidfile is not None:
@@ -364,21 +377,22 @@ class Arbiter(object):
         util._setproctitle("master [%s]" % self.proc_name)
         
         # manage workers
+        self.log.reopen_files()
+
         self.manage_workers() 
         
     def murder_workers(self):
         """\
         Kill unused/idle workers
         """
-        for (pid, worker) in list(self.WORKERS.items()):
+        for (pid, worker) in self.WORKERS.items():
             try:
-                diff = time.time() - os.fstat(worker.tmp.fileno()).st_ctime
-                if diff <= self.timeout:
+                if time.time() - worker.tmp.last_update() <= self.timeout:
                     continue
             except ValueError:
                 continue
 
-            self.log.critical("WORKER TIMEOUT (pid:%s)" % pid)
+            self.log.critical("WORKER TIMEOUT (pid:%s)", pid)
             self.kill_worker(pid, signal.SIGKILL)
         
     def reap_workers(self):
@@ -415,29 +429,28 @@ class Arbiter(object):
         if len(self.WORKERS.keys()) < self.num_workers:
             self.spawn_workers()
 
-        num_to_kill = len(self.WORKERS) - self.num_workers
-        for i in range(num_to_kill, 0, -1):
-            pid, age = 0, sys.maxint
-            for (wpid, worker) in self.WORKERS.iteritems():
-                if worker.age < age:
-                    pid, age = wpid, worker.age
+        workers = self.WORKERS.items()
+        workers.sort(key=lambda w: w[1].age)
+        while len(workers) > self.num_workers:
+            (pid, _) = workers.pop(0)
             self.kill_worker(pid, signal.SIGQUIT)
             
     def spawn_worker(self):
         self.worker_age += 1
         worker = self.worker_class(self.worker_age, self.pid, self.LISTENER,
-                                    self.app, self.timeout/2.0, self.cfg)
+                                    self.app, self.timeout/2.0,
+                                    self.cfg, self.log)
         self.cfg.pre_fork(self, worker)
         pid = os.fork()
         if pid != 0:
             self.WORKERS[pid] = worker
-            return
+            return pid
 
         # Process Child
         worker_pid = os.getpid()
         try:
             util._setproctitle("worker [%s]" % self.proc_name)
-            self.log.info("Booting worker with pid: %s" % worker_pid)
+            self.log.info("Booting worker with pid: %s", worker_pid)
             self.cfg.post_fork(self, worker)
             worker.init_process()
             sys.exit(0)
@@ -449,7 +462,7 @@ class Arbiter(object):
                 sys.exit(self.WORKER_BOOT_ERROR)
             sys.exit(-1)
         finally:
-            self.log.info("Worker exiting (pid: %s)" % worker_pid)
+            self.log.info("Worker exiting (pid: %s)", worker_pid)
             try:
                 worker.tmp.close()
                 self.cfg.worker_exit(self, worker)
