@@ -132,31 +132,27 @@ class Request(Message):
             idx = buf.getvalue().find("\r\n")
         self.parse_request_line(buf.getvalue()[:idx])
         rest = buf.getvalue()[idx+2:] # Skip \r\n
-        buf.truncate(0)
+        buf = StringIO()
         buf.write(rest)
        
         
         # Headers
-        pos = 0
         idx = buf.getvalue().find("\r\n\r\n")
 
         done = buf.getvalue()[:2] == "\r\n"
         while idx < 0 and not done:
-            pos = buf.tell() - 4
             self.get_data(unreader, buf)
-            idx = buf.getvalue()[pos:].find("\r\n\r\n")
+            idx = buf.getvalue().find("\r\n\r\n")
             done = buf.getvalue()[:2] == "\r\n"
              
         if done:
             self.unreader.unread(buf.getvalue()[2:])
             return ""
 
-        end = pos + idx
-        
-        self.headers = self.parse_headers(buf.getvalue()[:end])
+        self.headers = self.parse_headers(buf.getvalue()[:idx])
 
-        ret = buf.getvalue()[end+4:]
-        buf.truncate(0)
+        ret = buf.getvalue()[idx+4:]
+        buf = StringIO()
         return ret
     
     def parse_request_line(self, line):
@@ -171,7 +167,7 @@ class Request(Message):
 
         # URI
         self.uri = bits[1]
-        parts = urlparse.urlparse(bits[1])
+        parts = urlparse.urlsplit(bits[1])
         self.scheme = parts.scheme or ''
         self.host = parts.netloc or None
         if parts.port is None:
