@@ -45,7 +45,7 @@ class SyncWorker(base.Worker):
 
             # If our parent changed then we shut down.
             if self.ppid != os.getppid():
-                self.log.info("Parent changed, shutting down: %s" % self)
+                self.log.info("Parent changed, shutting down: %s", self)
                 return
             
             try:
@@ -81,6 +81,7 @@ class SyncWorker(base.Worker):
             util.close(client)
 
     def handle_request(self, req, client, addr):
+        environ = {}
         try:
             self.cfg.pre_request(self, req)
             resp, environ = wsgi.create(req, client, addr,
@@ -94,11 +95,17 @@ class SyncWorker(base.Worker):
                 self.log.info("Autorestarting worker after current request.")
                 self.alive = False
             respiter = self.wsgi(environ, resp.start_response)
-            for item in respiter:
-                resp.write(item)
-            resp.close()
-            if hasattr(respiter, "close"):
-                respiter.close()
+            try:
+                if isinstance(respiter, environ['wsgi.file_wrapper']):
+                    resp.write_file(respiter)
+                else:
+                    for item in respiter:
+                        resp.write(item)
+                self.log.access(resp, environ)
+                resp.close()
+            finally:
+                if hasattr(respiter, "close"):
+                    respiter.close()
         except socket.error:
             raise
         except Exception, e:
@@ -107,7 +114,7 @@ class SyncWorker(base.Worker):
             return
         finally:
             try:
-                self.cfg.post_request(self, req)
+                self.cfg.post_request(self, req, environ)
             except:
                 pass
 
